package com.example.ldap;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.example.dsp.ldap.InetOrgPerson;
import com.example.dsp.ldap.xmlbeans.InetOrgPersonDocument;
import com.example.dsp.ldap.xmlbeans.LdapKeyDocument;

public class DspLdapWrapper {
	private static InetOrgPersonDao ldapContact;

	static {
		// Load the Spring Application Context config file, and get the
		// core Data Access Object - ldapContact
		ApplicationContext factory = new ClassPathXmlApplicationContext(
				"com/example/ldap/springldap.xml");

		ldapContact = (InetOrgPersonDao) factory.getBean("ldapContact");
	}

	public static LdapKeyDocument[] createPerson(InetOrgPersonDocument[] persons) {
		// Map this call with zero or more persons to be created to
		// the ldapContact single person create method
		LdapKeyDocument[] keys = new LdapKeyDocument[persons.length];

		for (int i = 0, lim = persons.length; i < lim; i++) {
			keys[i] = ldapContact.createPerson(persons[i]);
		}

		return keys;
	}

	// DSP 3 allows update procedures to have an XMLBean parameter
	// However the update procedure can not be marked as Primary
	public static void updatePersonXMLBean(InetOrgPersonDocument[] persons) {
		for (InetOrgPersonDocument person : persons) {
			ldapContact.updatePerson(person);
		}
	}

	// DSP 3 only allows update procedures to be primary when its single
	// parameter is of type commonj.sdo.DataObject (i.e. an SDO type)
	public static void updatePersonSDO(InetOrgPerson[] persons) {
		// Map SDO/DSP DataObject (InetOrgPerson) to XMLBean XMLObject
		// (InetOrgPersonDocument)
		for (InetOrgPerson person : persons) {
			InetOrgPersonDocument personDoc = InetOrgPersonDocument.Factory
					.newInstance();
			personDoc.addNewInetOrgPerson();
			InetOrgPersonDocument.InetOrgPerson personXML = personDoc
					.getInetOrgPerson();
			personXML.setCommonName(person.getCommonName());
			personXML.setGivenName(person.getGivenName());
			personXML.setSurname(person.getSurname());
			personXML.setUserId(person.getUserId());

			ldapContact.updatePerson(personDoc);
		}
	}

	public static void deletePerson(InetOrgPersonDocument[] persons) {
		for (InetOrgPersonDocument person : persons) {
			ldapContact.deletePerson(person);
		}
	}

	public static InetOrgPersonDocument getPersonByUserId(String uid) {
		return ldapContact.findByPrimaryKey(uid);
	}

	public static InetOrgPersonDocument[] getAllPersons() {
		return ldapContact.findAll();
	}

	public static void main(String[] args) {
		InetOrgPersonDocument[] persons = getAllPersons();
		for (InetOrgPersonDocument p : persons) {
			System.out.println(p);
		}
	}
}
