package com.example.ldap;

import java.util.List;

import javax.naming.Name;

import org.springframework.ldap.core.ContextMapper;
import org.springframework.ldap.core.DirContextAdapter;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.ldap.core.DistinguishedName;
import org.springframework.ldap.core.LdapTemplate;
import org.springframework.ldap.core.support.AbstractContextMapper;
import org.springframework.ldap.filter.EqualsFilter;

import com.example.dsp.ldap.xmlbeans.InetOrgPersonDocument;
import com.example.dsp.ldap.xmlbeans.LdapKeyDocument;

public class InetOrgPersonDaoImpl implements InetOrgPersonDao {
	private LdapTemplate ldapTemplate;

	public void setLdapTemplate(LdapTemplate ldapTemplate) {
		this.ldapTemplate = ldapTemplate;
	}

	public LdapKeyDocument createPerson(InetOrgPersonDocument person) {
		DirContextAdapter context = new DirContextAdapter();
		mapToContext(person, context);
		ldapTemplate.bind(buildDn(person), context, null);

		// Populate XML Key document (LdapKeyDocument) with unique id of newly
		// created person
		LdapKeyDocument personKey = LdapKeyDocument.Factory.newInstance();
		personKey.addNewLdapKey();
		personKey.getLdapKey().setUserId(person.getInetOrgPerson().getUserId());
	
		return personKey;
	}

	public void updatePerson(InetOrgPersonDocument person) {
		DirContextOperations context = ldapTemplate.lookupContext(buildDn(person));
		mapToContext(person, context);
		ldapTemplate.modifyAttributes(context);
	}

	public void deletePerson(InetOrgPersonDocument person) {
		ldapTemplate.unbind(buildDn(person));
	}

	public InetOrgPersonDocument findByPrimaryKey(String uniqueIdentifier) {
		Name dn = buildDn(uniqueIdentifier);
		return (InetOrgPersonDocument) ldapTemplate.lookup(dn,
				getContextMapper());
	}

	@SuppressWarnings("unchecked")
	public InetOrgPersonDocument[] findAll() {
		EqualsFilter filter = new EqualsFilter("objectclass", "person");
		List list = ldapTemplate.search(DistinguishedName.EMPTY_PATH,
				filter.encode(), getContextMapper());
		InetOrgPersonDocument[] persons = new InetOrgPersonDocument[list.size()];
		list.toArray(persons);
		return persons;
	}

	protected ContextMapper getContextMapper() {
		return new InetOrgPersonContextMapper();
	}

	protected Name buildDn(InetOrgPersonDocument person) {
		return buildDn(person.getInetOrgPerson().getUserId());
	}

	protected Name buildDn(String uid) {
		DistinguishedName dn = new DistinguishedName();
		dn.add("uid", uid);
		return dn;
	}

	protected void mapToContext(InetOrgPersonDocument personDoc,
			DirContextOperations context) {
		InetOrgPersonDocument.InetOrgPerson person = personDoc.getInetOrgPerson();
		context.setAttributeValues("objectclass", new String[] { "top",
				"person", "organizationalPerson", "inetOrgPerson" });
		context.setAttributeValue("uid", person.getUserId());
		context.setAttributeValue("cn", person.getCommonName());
		context.setAttributeValue("sn", person.getSurname());
		context.setAttributeValue("givenname", person.getGivenName());
	}

	private static class InetOrgPersonContextMapper extends
			AbstractContextMapper {
		public Object doMapFromContext(DirContextOperations context) {
			InetOrgPersonDocument personDoc = InetOrgPersonDocument.Factory.newInstance();
			InetOrgPersonDocument.InetOrgPerson person = personDoc.addNewInetOrgPerson();
			person.setUserId(context.getStringAttribute("uid"));
			person.setCommonName(context.getStringAttribute("cn"));
			person.setSurname(context.getStringAttribute("sn"));
			person.setGivenName(context.getStringAttribute("givenname"));
			return personDoc;
		}
	}
}